package cn.iocoder.yudao.module.shipment.service.boxAirShipment;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.order.enums.OrderApprovalTypeResultEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxAirShipmentConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirShipmentDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxAirShipmentMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxAirShipment.*;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalCreateReqVO;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.TargetLogEnum.WITHDRAWAL;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 空运出货 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxAirShipmentServiceImpl extends AbstractService<BoxAirShipmentMapper, BoxAirShipmentDO> implements BoxAirShipmentService {

    @Resource
    private BoxAirShipmentMapper boxAirShipmentMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    private BoxOpLogService boxOpLogService;

    @Resource
    private BoxApprovalService approvalService;
    @Override
    public Long createBoxAirShipment(BoxAirShipmentCreateReqVO createReqVO) {
        List<BoxAirShipmentDO> boxAirShipmentDOS = boxAirShipmentMapper.selectList(BoxAirShipmentDO::getShipmentId, createReqVO.getShipmentId());
        if (CollectionUtil.isNotEmpty(boxAirShipmentDOS)) {
            boxAirShipmentMapper.deleteBatchIds(boxAirShipmentDOS.stream().map(BoxAirShipmentDO::getId).collect(Collectors.toList()));
        }
        // 插入
        BoxAirShipmentDO boxAirShipment = BoxAirShipmentConvert.INSTANCE.convert(createReqVO);
        boxAirShipmentMapper.insert(boxAirShipment);
        // 返回
        commitUpdate(createReqVO.getOperateType(),createReqVO.getShipmentId());
        return boxAirShipment.getId();
    }

    @Override
    public void updateBoxAirShipment(BoxAirShipmentUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxAirShipmentExists(updateReqVO.getId());
        // 更新
        BoxAirShipmentDO updateObj = BoxAirShipmentConvert.INSTANCE.convert(updateReqVO);
        commitUpdate(updateReqVO.getOperateType(),updateReqVO.getShipmentId());
        boxAirShipmentMapper.updateById(updateObj);
    }

    @Override
    public void recallBoxAirShipment(BoxAirShipmentUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxAirShipmentExists(updateReqVO.getId());
        // 撤回
        commitRecall(updateReqVO.getShipmentId(),updateReqVO.getRemark());
    }

    private void commitUpdate(Integer operateType, Long shipmentId) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            int airShipmentStatus = boxDO.getAirShipmentStatus();
            AirShipmentStatusEnum shipment = AirShipmentStatusEnum.SHIPMENT;

            if(airShipmentStatus != shipment.getStatus()) {
                //出货审核
                //createApproval(shipmentId, copyUserId);
                createBoxShipment(boxDO, shipmentId);
            }
        }
        boxService.updateById(boxDO);
    }

    private void commitRecall(Long shipmentId,String remark) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        int airShipmentStatus = boxDO.getAirShipmentStatus();
        AirShipmentStatusEnum shipment = AirShipmentStatusEnum.SHIPMENT;

        if (airShipmentStatus == shipment.getStatus()) {
            recallBoxShipment(boxDO, shipmentId,remark);
        }
    }
    //撤回空运出货
    private void recallBoxShipment(BoxDO boxDO, Long shipmentId,String remark) {

        //返回成功，修改为预装中的状态
        boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
        boxDO.setToWarehouseStatus(TWStatusEnum.NO_TO_WAREHOUSE.getTwStatus());
        boxDO.setClStatus(ClStatueEnum.NO_CLEARANCE.getClStatus());
        boxDO.setSapStatus(SapStatueEnum.NO_ARRIVE.getSapStatus());
        boxDO.setCdStatus(CdStatueEnum.NO_CLEARANCE_DOCUMENTS.getCdStatus());
        boxDO.setFlyStatus(FlyStatueEnum.NO_FLYING.getDtStatus());
        boxDO.setBlStatus(BlStatueEnum.NO_LADING_BILL.getBlStatus());
        boxDO.setDcStatus(DcStatueEnum.UNCUSTOMED.getDcStatus());
        boxDO.setCheckoutStatus(AirCheckoutStatusEnum.UNCHECKOUT.getStatus());
        boxDO.setAirShipmentStatus(AirShipmentStatusEnum.UNSHIPMENT.getStatus());
        boxDO.setShipmentStatusAir(BoxAirStatusEnum.TALLY_COMPLETE.getStatus());
        String remarkResult = remark != null && !remark.isEmpty() ? String.format(" 备注【%s】", remark) : "";
        String text = AirShipmentStatusEnum.SHIPMENT_BACK_RECALL_SUCCESS.getText() + remarkResult;;
        String textEn = AirShipmentStatusEnum.SHIPMENT_BACK_RECALL_SUCCESS.getTextEn() + remarkResult;
        //添加操作日志
        boxOpLogService.createBoxOpLog(boxDO.getId(), AirShipmentStatusEnum.SHIPMENT_BACK_RECALL_SUCCESS.getStatus(),
                text, textEn);

        //反审后修改订单为出货状态
        boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.SHIPMENT.getValue(), null,
                BoxAirStatusEnum.TALLY_COMPLETE.getStatus(), 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_back_recall.getType(), 5).getDesc(), Integer.parseInt(boxDO.getTransportType()));

        //反审后清除订单表的提单号
//                    boxService.clearOrderTidanNo(shipmentId, false);

        //业绩变更
        boxService.addTargetLog(shipmentId, WITHDRAWAL, new Date());
        //删除已到港订单
        boxService.deleteArrivalOrderByShipment(shipmentId);
        //删除已清关订单
        boxService.deleteClearanceOrderByShipment(shipmentId);
        boxService.updateById(boxDO);
    }


    //去掉审核直接出货
    private void createBoxShipment(BoxDO boxDO, Long shipmentId) {
        boxDO.setAirShipmentStatus(AirShipmentStatusEnum.SHIPMENT.getStatus());
        //修改为出货审核后生成提单号
        //boxService.generateBillLadingNo(shipmentId, true, false);
        //修改订单状态
        boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.SHIPMENT.getValue(),
                null, BoxAirStatusEnum.AIR_SHIPPED.getStatus(), 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_noAudit.getType(), 5).getDesc(), Integer.parseInt(boxDO.getTransportType()));

        boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.AIR_OUT_SHIPMENT, "");
    }

    private void createApproval(Long shipmentId, String[] copyUserId) {
        BoxApprovalCreateReqVO boxApprovalCreateReqVO = new BoxApprovalCreateReqVO();
        boxApprovalCreateReqVO.setShipmentId(shipmentId);
        boxApprovalCreateReqVO.setApplyReason("出货审核");
        boxApprovalCreateReqVO.setCopyUserId(copyUserId);
        boxApprovalCreateReqVO.setApprovalStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        boxApprovalCreateReqVO.setApprovalType(BoxApprovalTypeEnum.AIR_SHIPMENT.getType());
        approvalService.createBoxApproval(boxApprovalCreateReqVO);
    }

    @Override
    public void deleteBoxAirShipment(Long id) {
        // 校验存在
        this.validateBoxAirShipmentExists(id);
        // 删除
        boxAirShipmentMapper.deleteById(id);
    }

    private void validateBoxAirShipmentExists(Long id) {
        if (boxAirShipmentMapper.selectById(id) == null) {
            throw exception(BOX_AIR_SHIPMENT_NOT_EXISTS);
        }
    }

    @Override
    public BoxAirShipmentDO getBoxAirShipment(Long id) {
        return boxAirShipmentMapper.selectById(id);
    }

    @Override
    public List<BoxAirShipmentDO> getBoxAirShipmentList(Collection<Long> ids) {
        return boxAirShipmentMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxAirShipmentDO> getBoxAirShipmentPage(BoxAirShipmentQueryVO query, PageVO page) {
        return boxAirShipmentMapper.selectPage(page, query);
    }

    @Override
    public List<BoxAirShipmentDO> getBoxAirShipmentList(BoxAirShipmentQueryVO query) {
        return boxAirShipmentMapper.selectList(query);
    }

    @Override
    public BoxAirShipmentDO getAirShipmentByShipmentId(Long shipmentId) {
        return boxAirShipmentMapper.selectOne(BoxAirShipmentDO::getShipmentId,shipmentId);
    }

    private void checkBoxApproval(Long shipmentId){
        Long aLong = approvalService.selectCount(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalStatus, 0));
        if (aLong != null && aLong > 0L) {
            throw exception(BOX_APPROVAL_EXIST);
        }
    }
}
